package com.example.sqlrun.service.impl;

import com.example.sqlrun.fixSql.MakePartitionSql;
import com.example.sqlrun.fixSql.MakeTableSpace;
import com.example.sqlrun.entity.SqlNeedEntity;
import com.example.sqlrun.myEnum.SpaceEnum;
import com.example.sqlrun.service.SpaceAndPartitionService;
import com.shanghaigeography.Util.DateUtils;
import java.sql.SQLException;
import java.text.ParseException;
import java.time.YearMonth;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import java.sql.Statement;

/**
 * @ClassName: SpaceAndPartitionServiceImpl
 *
 * @description: TODO
 * @author: sbj
 * @time: 2023/12/5 15:55 @Version: 1.0
 */
@Service
public class SpaceAndPartitionServiceImpl implements SpaceAndPartitionService {
  @Resource(name = "statement")
  private Statement statement;

  @Override
  public void runSQL(SqlNeedEntity sqlNeedEntity) throws ParseException, SQLException {
    // 获取需要创建表空间的sql
    List<Map> tableSpaces =
        MakeTableSpace.makeTableSpaceSql(
            sqlNeedEntity.getTableSpaceNamePrefix(),
            sqlNeedEntity.getTableSpaceFileNamePrefix(),
            sqlNeedEntity.getTableSpacePath(),
            sqlNeedEntity.getInitSize(),
            sqlNeedEntity.getNextSize(),
            sqlNeedEntity.getMaxSize(),
            sqlNeedEntity.getSpaceLastDate(),
            sqlNeedEntity.getWantedTableSize());
    for (Map tableSpace : tableSpaces) {
      Date date = (Date) tableSpace.get(SpaceEnum.DATE.getName());
      List<String> partitionSql =
          MakePartitionSql.makePartitionSql(
              sqlNeedEntity.getTableSpaceNamePrefix(),
              sqlNeedEntity.getPartitionNamePrefix(),
              tableSpace.get(SpaceEnum.TABLESPACENAME.getName()).toString(),
              date,
              YearMonth.of(date.getYear() + 1900, date.getMonth() + 1).lengthOfMonth());
      statement.execute(tableSpace.get(SpaceEnum.TABLESPACE_SQL.getName()).toString());
      partitionSql.forEach(
          i -> {
            try {
              statement.addBatch(i);
            } catch (SQLException throwables) {
              throwables.printStackTrace();
            }
          });
      statement.executeBatch();
      statement.clearBatch();
    }
  }

  @Override
  public List<SqlNeedEntity> getSqlNeed() {
    return null;
  }

  @Override
  public String createTableSpace(Map<String, String> paramBody) {
    try {
      String tableSpaceNamePrefix = paramBody.get("tableSpaceName");
      String tableSpaceFileNamePrefix = paramBody.get("tableSpaceFileName");
      String tableSpacePath = paramBody.get("tableSpacePath");
      Integer initSize = Integer.valueOf(paramBody.get("initSize"));
      Integer nextSize = Integer.valueOf(paramBody.get("nextSize"));
      Integer maxSize = Integer.valueOf(paramBody.get("maxSize"));
      String startDate = paramBody.get("startDate");
      Integer wanted = Integer.valueOf(paramBody.get("wanted"));
      List<Map> maps =
          MakeTableSpace.makeTableSpaceSql(
              tableSpaceNamePrefix,
              tableSpaceFileNamePrefix,
              tableSpacePath,
              initSize,
              nextSize,
              maxSize,
              startDate,
              wanted);
      for (Map map : maps) {
        statement.execute(map.get(SpaceEnum.TABLESPACE_SQL.getName()).toString());
      }
      Map map = maps.get(maps.size() - 1);
      return "最后一个表空间时间"
          + map.get(SpaceEnum.DATE.getName())
          + "\n"
          + "最后一个表空间名称"
          + map.get(SpaceEnum.TABLESPACENAME.getName());
    } catch (Exception e) {
      return "error" + e.getMessage();
    }
  }

  @Override
  public String createTablePartition(Map<String, String> param) {
    try {
      String tableName = param.get("tableName");
      String partitionNamePrefix = param.get("partitionNamePrefix");
      String tableSpaceName = param.get("tableSpaceName");
      Date startDate = DateUtils.stringToDate(param.get("startDate"), "yyyy-MM-dd");
      Integer wantedDay = null;
      if (StringUtils.isBlank(param.get("wantedDay"))){
        Calendar instance = Calendar.getInstance();
        instance.setTime(startDate);
        wantedDay = instance.getActualMaximum(Calendar.DAY_OF_MONTH) - instance.get(Calendar.DAY_OF_MONTH) + 1;
      }else {
        wantedDay = Integer.valueOf(param.get("wantedDay"));
      }
      List<String> partitionSql = MakePartitionSql
          .makePartitionSql(tableName, partitionNamePrefix, tableSpaceName, startDate, wantedDay);
      for (String s : partitionSql) {
        statement.addBatch(s);
      }
      statement.executeBatch();
      statement.clearBatch();
      return "success";
    } catch (Exception e) {
      return "error" + e.getMessage();
    }
  }
}
